@@ -19,6 +19,7 @@
 """Blackbox tests for Dulwich commands."""
 
 import tempfile
+import shutil
 
 from dulwich.repo import (
     Repo,
@@ -34,20 +35,22 @@ class GitReceivePackTests(BlackboxTestCase):
     def setUp(self):
         super(GitReceivePackTests, self).setUp()
         self.path = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, self.path)
         self.repo = Repo.init(self.path)
 
     def test_basic(self):
         process = self.run_command("dul-receive-pack", [self.path])
-        (stdout, stderr) = process.communicate("0000")
-        self.assertEqual('', stderr)
-        self.assertEqual('0000', stdout[-4:])
+        (stdout, stderr) = process.communicate(b"0000")
+        self.assertEqual(b'0000', stdout[-4:])
         self.assertEqual(0, process.returncode)
 
     def test_missing_arg(self):
         process = self.run_command("dul-receive-pack", [])
         (stdout, stderr) = process.communicate()
-        self.assertEqual('usage: dul-receive-pack <git-dir>\n', stderr)
-        self.assertEqual('', stdout)
+        self.assertEqual(
+            [b'usage: dul-receive-pack <git-dir>'],
+            stderr.splitlines()[-1:])
+        self.assertEqual(b'', stdout)
         self.assertEqual(1, process.returncode)
 
 
@@ -57,11 +60,14 @@ class GitUploadPackTests(BlackboxTestCase):
     def setUp(self):
         super(GitUploadPackTests, self).setUp()
         self.path = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, self.path)
         self.repo = Repo.init(self.path)
 
     def test_missing_arg(self):
         process = self.run_command("dul-upload-pack", [])
         (stdout, stderr) = process.communicate()
-        self.assertEqual('usage: dul-upload-pack <git-dir>\n', stderr)
-        self.assertEqual('', stdout)
+        self.assertEqual(
+            [b'usage: dul-upload-pack <git-dir>'],
+            stderr.splitlines()[-1:])
+        self.assertEqual(b'', stdout)
         self.assertEqual(1, process.returncode)
